package com.miaozhen.mzanalytics;

import android.app.Activity;
import android.content.Context;
import android.util.Log;

import androidx.annotation.NonNull;

import org.json.JSONObject;

import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** MzanalyticsPlugin */
public class MzanalyticsPlugin implements FlutterPlugin, MethodCallHandler {
  private  static Context applicationContext;

  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "mzanalytics");
    channel.setMethodCallHandler(this);
    applicationContext = flutterPluginBinding.getApplicationContext();
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "mzanalytics");
    channel.setMethodCallHandler(new MzanalyticsPlugin());
    Activity activity = registrar.activity();
    applicationContext  = activity.getApplicationContext();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
 try {
            if (call.method.equals("android_initEvent")) {
                Log.d("lincoln",call.method+  "  lincoln map ");
                EventUtils.initEvent(applicationContext);
            } else if (call.method.equals("android_sendBaseEvent")) {
                Map<String, String> map = (Map<String, String>) call.arguments;
                JSONObject baseObject = new JSONObject(map);
                Log.d("lincoln",call.method+  "  lincoln map "+ map.toString()+ " "+baseObject.toString());
                EventUtils.BaseEvent(applicationContext, baseObject);
            } else  if (call.method.equals("android_pageEvent")) {
                Map<String, String> map = (Map<String, String>) call.arguments;
                JSONObject baseObject = new JSONObject(map);
                Log.d("lincoln",call.method+  "  lincoln map " +map.toString()+ " "+baseObject.toString());
                EventUtils.pageEvent(applicationContext, baseObject);
            } else if (call.method.equals("android_seniorEvent")) {
                Map<String, String> map = (Map<String, String>) call.arguments;
                JSONObject baseObject = new JSONObject(map);
                Log.d("lincoln",call.method+  "  lincoln map " +map.toString()+ " "+baseObject.toString());
                EventUtils.SeniorEvent(applicationContext, baseObject);
            } else if (call.method.equals("android_heartEvent")) {
                Log.d("lincoln",call.method+  "  lincoln map ");
                EventUtils.heartEvent(applicationContext);
            }else if(call.method.equals("android_getCid")){
                String cid = EventUtils.getCid(applicationContext);
                result.success(cid);
            }else if(call.method.equals("android_getOaid")){
                String oaid = EventUtils.getOaid(applicationContext);
                result.success(oaid);
            }
            else {
                result.notImplemented();
            }
        } catch (Throwable e) {
            e.printStackTrace();
        }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }
}
