import 'dart:async';
import 'package:built_collection/built_collection.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter/services.dart';
import 'dart:io';
import 'package:path_provider/path_provider.dart';
import 'dart:convert';

class Mzanalytics {
  static const MethodChannel _channel = const MethodChannel('mzanalytics');

  static Future<void> ios_appid(String aid) async {
    try {
      _channel.invokeMethod('ios_appid', {'appid': aid});
    } on PlatformException {}
  }

  static Future<void> ios_trackPageView(var parameter) async {
    try {
      _channel.invokeMethod('ios_trackPageView', parameter);
    } on PlatformException {}
  }

  static Future<void> ios_trackEvent(var parameter) async {
    try {
      _channel.invokeMethod('ios_trackEvent', parameter);
    } on PlatformException {}
  }

  static Future<void> ios_trackSiteCustomEvent(var parameter) async {
    try {
      _channel.invokeMethod('ios_trackSiteCustomEvent', parameter);
    } on PlatformException {}
  }

  static Future<void> get ios_trackPulse async {
    try {
      _channel.invokeMethod('ios_trackPulse');
    } on PlatformException {}
  }

  static Future<String> get ios_getCid async {
    try {
      final String cid = await _channel.invokeMethod('ios_getCid');
      return cid;
    } on PlatformException {}
  }

  static Future<void> ios_setEnableLog(bool parameter) async {
    try {
      _channel.invokeMethod('ios_setEnableLog', {'Enable': parameter});
    } on PlatformException {}
  }

  static Future<void> ios_handleOpenURL(String url) async {
    try {
      _channel.invokeMethod('ios_handleOpenURL', {'url': url});
    } on PlatformException {}
  }

  static Future<void> android_initEvent() async {
    try {
      _channel.invokeMethod('android_initEvent');
    } on PlatformException catch (e) {}
  }

  static Future<void> android_sendBaseEvent(var parameter) async {
    try {
      _channel.invokeMethod('android_sendBaseEvent', parameter);
    } on PlatformException catch (e) {}
  }

  static Future<void> android_pageEvent(var parameter) async {
    try {
      _channel.invokeMethod('android_pageEvent', parameter);
    } on PlatformException catch (e) {}
  }

  static Future<void> android_seniorEvent(var parameter) async {
    try {
      _channel.invokeMethod('android_seniorEvent', parameter);
    } on PlatformException catch (e) {}
  }

  static Future<void> android_heartEvent() async {
    try {
      _channel.invokeMethod('android_heartEvent');
    } on PlatformException catch (e) {}
  }

  static Future<String> android_getOaid() async {
    try {
      return _channel.invokeMethod('android_getOaid');
    } on PlatformException catch (e) {}
  }

  static Future<String> android_getCid() async {
    try {
      return _channel.invokeMethod('android_getCid');
    } on PlatformException catch (e) {}
  }

  //IO操作,存取自定义参数
  static Future<File> _getLocalFile() async {
    // get the path to the document directory.
    String dir = (await getApplicationDocumentsDirectory()).path;
    return new File('$dir/self_event.txt');
  }

  static Future<Map> readParams() async {
    try {
      File file = await _getLocalFile();
      String contents = await file.readAsString();
      Map result = json.decode(contents);
      return result;
    } on FileSystemException {
      return null;
    }
  }

  static Future<Null> writeParams(var parameter) async {
    try {
      print("开始准备写入参数:");
      var opreate_params = new Map<String, dynamic>.from(parameter);
      for (var key in parameter.keys) {
        if (key.startsWith('cd') || key.startsWith('cm')) {
          print(key);
        } else {
          opreate_params.remove(key);
        }
      }
      String jsonString = json.encode(opreate_params);
      print("最终写入参数:" + jsonString);

      // write the variable as a string to the file
      await (await _getLocalFile()).writeAsString('$jsonString');
    } on FileSystemException {}
  }
}

class MZNavigationHistoryObserver extends NavigatorObserver {
  /// A list of all the past routes
  List<Route<dynamic>> _history = <Route<dynamic>>[];

  /// Gets a clone of the navigation history as an immutable list.
  get history => BuiltList<Route<dynamic>>.from(_history);

  /// Gets the top route in the navigation stack.
  get top => _history.last;

  /// A list of all routes that were popped to reach the current.
  List<Route<dynamic>> _poppedRoutes = <Route<dynamic>>[];

  /// Gets a clone of the popped routes as an immutable list.
  get poppedRoutes => BuiltList<Route<dynamic>>.from(_poppedRoutes);

  /// Gets the next route in the navigation history, which is the most recently popped route.
  get next => _poppedRoutes.last;

  /// A stream that broadcasts whenever the navigation history changes.
  StreamController _historyChangeStreamController =
      StreamController.broadcast();

  /// Accessor to the history change stream.
  get historyChangeStream => _historyChangeStreamController.stream;

  static final MZNavigationHistoryObserver _singleton =
      MZNavigationHistoryObserver._internal();
  MZNavigationHistoryObserver._internal();
  factory MZNavigationHistoryObserver() {
    return _singleton;
  }

  @override
  void didPop(Route<dynamic> route, Route<dynamic> previousRoute) {
    _poppedRoutes.add(_history.last);
    _history.removeLast();
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.pop,
      newRoute: route,
      oldRoute: previousRoute,
    ));
  }

  @override
  void didPush(Route<dynamic> route, Route<dynamic> previousRoute) {
    _history.add(route);
    _poppedRoutes.remove(route);
    try {
      if (route.settings.name != null && route.settings.name != '/') {
        print('sending...');
        Map selfparams = new Map();
        // Future<Map> value = Mzanalytics.readParams();
        Mzanalytics.readParams().then((Map value) {
          print("读取的自定义事件:" + value.toString());
          if(value != null){
            selfparams = value;   
          }
          //{'pageTitle': route.settings.name;CustomDimMetrics:字典}; {'pageTitle': route.settings.name;字典}
          if (Platform.isIOS) {
            //ios相关代码
            Map customDimMetricsMap = new Map();
            for (String key in selfparams.keys) {
              if (key.startsWith('cd')) {
                print(key);
                String newkey = key.replaceFirst("cd", "customDimension");
                var value = selfparams[key];
                customDimMetricsMap[newkey]=value;
              } else if(key.startsWith('cm')){
                String newkey = key.replaceFirst("cm", "customMetrics");
                var value = selfparams[key];
                customDimMetricsMap[newkey]=value;
              }
            }
            Map sendParams = new Map();
            sendParams['pageTitle'] = route.settings.name;
            sendParams['CustomDimMetrics'] = customDimMetricsMap;

            print('测试ios发送的数据：' + sendParams.toString());

            Mzanalytics.ios_trackPageView(sendParams);
          } else if (Platform.isAndroid) {
            //android相关代码
            selfparams['dt'] = route.settings.name;
            print('准备发送的数据：' + selfparams.toString());
            Mzanalytics.android_pageEvent(selfparams);
          }
        });
      }
    } on PlatformException {}
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.push,
      newRoute: route,
      oldRoute: previousRoute,
    ));
  }

  @override
  void didRemove(Route<dynamic> route, Route<dynamic> previousRoute) {
    _history.remove(route);
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.remove,
      newRoute: route,
      oldRoute: previousRoute,
    ));
  }

  @override
  void didReplace({Route<dynamic> newRoute, Route<dynamic> oldRoute}) {
    int oldRouteIndex = _history.indexOf(oldRoute);
    _history.replaceRange(oldRouteIndex, oldRouteIndex + 1, [newRoute]);
    _historyChangeStreamController.add(HistoryChange(
      action: NavigationStackAction.replace,
      newRoute: newRoute,
      oldRoute: oldRoute,
    ));
  }
}

/// A class that contains all data that needs to be broadcasted through the history change stream.
class HistoryChange {
  HistoryChange({this.action, this.newRoute, this.oldRoute});

  final NavigationStackAction action;
  final Route<dynamic> newRoute;
  final Route<dynamic> oldRoute;
}

enum NavigationStackAction { push, pop, remove, replace }
